/*
 * GLUT CUBE DEMO
 * original in C by Wouter Lievens
 * adapted to Processing by Tom Van Cutsem
 *
 * This demo renders a 3D cube, while the camera rotates around it.
 */
import processing.opengl.*;

void setup() {
  // third argument = the 'renderer' to use
  // by default, Java 2D is used, now we use OpenGL
  size(600, 600, OPENGL);
  noStroke(); // don't draw lines to connect the vertices of the cube
  colorMode(RGB, 1.0); // colors are in RGB format, with values between 0 and 1.
}

/**
 * Draws a vertex of the cube, and sets the color depending on the vertex
 * location.
 *
 * Ie. consider plotting at (1, 0, 0):
 *  we set the color to (R,G,B):
 *   (1 * 1/2 + 1/2, 0 * 1/2 + 1/2, 0 * 1/2 + 1/2) 
 * = (1/2 + 1/2, 1/2, 1/2)
 * = (1, 1/2, 1/2)
 *
 * RGB colors are specified using floats ranging from
 * (0,0,0) = black, to (1,1,1) = white
 */
float factor = 0.50f;
void drawVertex(float x, float y, float z) {
  fill(x * factor + 0.5, y * factor + 0.5, z * factor + 0.5);
  vertex(x, y, z);
} 

/**
 * Draws a cube where all planes are facing outside.
 */
void drawCube() {
  beginShape(QUADS);
  {
    // bottom plane
    drawVertex(-1, -1, -1);
    drawVertex(-1, +1, -1);
    drawVertex(+1, +1, -1);
    drawVertex(+1, -1, -1);
    // top plane
    drawVertex(-1, -1, +1);
    drawVertex(+1, -1, +1);
    drawVertex(+1, +1, +1);
    drawVertex(-1, +1, +1);
    // front plane
    drawVertex(-1, -1, -1);
    drawVertex(+1, -1, -1);
    drawVertex(+1, -1, +1);
    drawVertex(-1, -1, +1);
    // back plane
    drawVertex(-1, +1, -1);
    drawVertex(-1, +1, +1);
    drawVertex(+1, +1, +1);
    drawVertex(+1, +1, -1);
    // left plane
    drawVertex(-1, -1, -1);
    drawVertex(-1, -1, +1);
    drawVertex(-1, +1, +1);
    drawVertex(-1, +1, -1);
    // right plane
    drawVertex(+1, -1, -1);
    drawVertex(+1, +1, -1);
    drawVertex(+1, +1, +1);
    drawVertex(+1, -1, +1);
  }
  endShape();
}

/*
 * Angle variable used to remember the camera's angle, as it orbits the
 * origin.
 */
float angle = 0;
/*
 * The radius at which the camera orbits the origin.
 */
float radius = 4;
/*
 * The amplitude of the tilting on the Z axis.
 */
float tilt = 2;
/*
 * The speed factor (relative to the orbital speed) at which tilting occurs.
 */
float tiltspeed = 4;

/*************************
 * Extra motion variables
 *************************/
float scalefactor = 1.0; // zooming factor
float a = 0; // rotation parameter

void draw() {
  // clear the screen
  background(0);  
  
  // set modelview matrix to identity matrix
  // apparently, Processing does this automatically for us
  //resetMatrix();
    
  // by using beginCamera() and endCamera(), matrix-manipulation
  // functions such as translate, scale, etc. will affect the
  // projection matrix (which controls how the scene is rendered)
  // rather than the modelview matrix (which controls the cube itself)
  // apparently, again, Processing automatically resets the projection matrix
  /*beginCamera();
    // reset the matrix to the identity matrix
    resetMatrix();
  endCamera();*/
      
  // load a perspective matrix
  perspective(90,   // the vertical field of view, set to 90 degrees
              1.0,  // the aspect ratio of width/height
              0.5,  // the 'near' clipping plane
              100); // the 'far' clipping plane
  
  // setup the camera
  camera(cos(angle) * radius, // the x coordinate of the Eye
         sin(angle) * radius, // the y coordinate of the Eye
         cos(tiltspeed * angle) * tilt, // the z coordinate of the Eye
         0, 0, 0, // the x, y, z target that the camera looks at
         0, 0, 1); // the orientation of the world, [0, 0, 1] means 'Z is UP'
  
  // increment the angle variable, this makes the camera turn
  angle += 0.005;
  
  // in Processing, apparently, it is important that you manipulate
  // the modelview matrix after you have initialised the camera
  
  /**********************************
   * make cube turn counterclockwise
   **********************************/
  translate(0,-cos(a),sin(a));
  a += 0.01;
 
  /*******************
   * make cube rotate
   *******************/
  rotateX(cos(a)*radius);
  rotateY(sin(a)*radius);
  rotateZ(cos(tiltspeed * angle) * tilt);
  
  /*********************
   * Zooming in and out
   *********************/
  scale(cos(scalefactor), cos(scalefactor), cos(scalefactor));
  scalefactor += 0.01;
    
  // draw the cube
  drawCube();
}
