/* Author: Brian J. Ross
   October 1999
   
   Adapted by Tom Van Cutsem
   
   Draw 3D Sierpinski Pyramids to level i. 
   The unit pyramid is based upon the local origin sitting in its middle.
   The recursive cases presume the sub-pyramids are scaled 1/2, and then
   translated to new loci around bigger triangle position. */
import processing.opengl.*;

// position and rotation speed variables.
float X_Rot   = 0.9f;
float Y_Rot   = 0.0f;
float X_Speed = 0.01f;
float Y_Speed = 0.01f;
float Z_Off   =-5.0f;

void setup() {
  size(600, 600, OPENGL);
  noStroke(); // don't draw lines to connect vertices
  colorMode(RGB, 1.0); // colors are in RGB format with values between 0 and 1.
  
  beginCamera();
    ortho(-125.0, 125.0, -125.0, 125.0, -125.0, 125.0);
    rotate(15.0, 1.0, 0.0, 0.0);
    rotate(45.0, 0.0, 1.0, 0.0);
  endCamera();
}

void recursive_tri(int i) {
  if (i <= 1) {  /* Base case: draw a pyramid */
    beginShape(TRIANGLES);
      fill(1.0f, 0.0f, 0.0f);
      vertex(-80.0, -80.0, 80.0);
      fill(0.0f, 1.0f, 0.0f);
      vertex(80.0, -80.0, 80.0);
      fill(0.0f, 0.0f, 1.0f);
      vertex(0.0, 80.0, 0.0);

      fill(1.0f, 0.0f, 0.0f);
      vertex(80.0, -80.0, 80.0);
      fill(0.0f, 1.0f, 0.0f);
      vertex(0.0, -80.0, -80.0);
      fill(0.0f, 0.0f, 1.0f);
      vertex(0.0, 80.0, 0.0);

      fill(1.0f, 0.0f, 0.0f);
      vertex(0.0, -80.0, -80.0);
      fill(0.0f, 1.0f, 0.0f);
      vertex(-80.0, -80.0, 80.0);
      fill(0.0f, 0.0f, 1.0f);
      vertex(0.0, 80.0, 0.0);

      fill(1.0f, 0.0f, 0.0f);
      vertex(-80.0, -80.0, 80.0);
      fill(0.0f, 1.0f, 0.0f);
      vertex(80.0, -80.0, 80.0);
      fill(0.0f, 0.0f, 1.0f);
      vertex(0.0, -80.0, -80.0);
    endShape();
  } else {
    --i;
    pushMatrix(); /* draw lower front left */
    translate(-40.0, -40.0, 40.0);
    scale(0.5, 0.5, 0.5);
    recursive_tri(i);
    popMatrix();

    pushMatrix();	 /* draw lower front right */
    translate(40.0, -40.0, 40.0);
    scale(0.5, 0.5, 0.5);
    recursive_tri(i);
    popMatrix();

    pushMatrix();	 /* draw lower back middle */
    translate(0.0, -40.0, -40.0);
    scale(0.5, 0.5, 0.5);
    recursive_tri(i);
    popMatrix();

    pushMatrix();	 /* draw top-middle */
    translate(0.0, 40.0, 0.0);
    scale(0.5, 0.5, 0.5);
    recursive_tri(i);
    popMatrix();
  }
}

// depth variable

int recDepth = 6;

void draw() {
  background(0); // clear screen
  
  // Reset to 0,0,0; no rotation, no scaling.
  resetMatrix();
  
  // Move the object back from the screen.
  translate(0.0f,0.0f,Z_Off);

  // Rotate the calculated amount.
  rotate(X_Rot,1.0f,0.0f,0.0f);
  rotate(Y_Rot,0.0f,1.0f,0.0f);
	 
  // draw the sierpinski triangles
  recursive_tri(recDepth);
	 	 
  // Now let's do the motion calculations.
  X_Rot+=X_Speed;
  Y_Rot+=Y_Speed;
}

void keyPressed() {
  if (key == CODED) {
    // SPECIAL KEY PRESS
    switch (keyCode) {
      case KeyEvent.VK_PAGE_UP: // move into the distance.
        Z_Off -= 0.05f;
        break;
      case KeyEvent.VK_PAGE_DOWN: // move closer.
        Z_Off += 0.05f;
        break;
      case UP: // decrease x rotation speed;
        X_Speed -= 0.01f;
        break;
      case DOWN: // increase x rotation speed;
        X_Speed += 0.01f;
        break;
      case LEFT: // decrease y rotation speed;
        Y_Speed -= 0.01f;
        break;
      case RIGHT: // increase y rotation speed;
        Y_Speed += 0.01f;
        break;
      }
  } else {
    // NORMAL KEY PRESS
    switch (key) {      
      case '+':
        recDepth++;
        break;
      case '-': {
        if (recDepth > 1) { recDepth--; }
        break;
      }
    }
  }
}
